# -*- coding:utf-8 -*-
# @Time     :2022/7/1 11:20
# @Author   :CHNJX
# @File     :database_conn.py
# @Desc     :获取数据库链接

import pymysql


class DatabaseConn:
    conn = None
    cursor = None

    def __init__(self, database_config):
        """
        :param database_config:
            example:
            database_config = {
                'host': 'xxx.cn',
                'port': 3306,
                'user': 'root',
                'password': '123456',
                'database': 'test',
                'autocommit': True
            }
        """
        self.create_conn(database_config)

    def create_conn(self, database_config):
        if not self.conn:
            self.conn = pymysql.connect(charset='utf8', **database_config)
            self.cursor = self.conn.cursor()

    def close_conn(self):
        if self.conn:
            self.conn.close()

    def excuse_sql(self, sql_str, params=None):
        self.conn.ping(reconnect=True)
        self.cursor.execute(sql_str, params)
        # 判断是否为查询语句
        try:
            self.conn.ping(reconnect=True)
            self.cursor.execute(sql_str, params)
            # 判断是否为查询语句
            if 'select' in sql_str.lower():
                return self.cursor.fetchone()
            else:
                self.conn.commit()
                return self.cursor.rowcount
        except Exception as e:
            print(f"Error in executing sql: {str(e)}")
            self.conn.rollback()

    def excuse_sql_with_all(self, sql_str, params=None):
        """查询语句获取全量数据"""
        try:
            self.conn.ping(reconnect=True)
            self.cursor.execute(sql_str, params)
            return self.cursor.fetchall()
        except Exception as e:
            print(f"Error in executing sql: {str(e)}")
            self.conn.rollback()



# -*- coding:utf-8 -*-
# @Time     :2023/2/7 12:38
# @Author   :CHNJX
# @File     :command.py
# @Desc     :命令工具
import subprocess
import sys
from os.path import dirname, exists
import click as click

from api_driver.har_parser import HarParser

sys.path.append(dirname(sys.path[0]))

from api_driver.project_generator import ProjectGenerator
from api_driver.swagger_generate import SwaggerGenerator

group = click.Group()


@click.command('start_project')
@click.option('-n', '--project-name', required=True, help='project name')
def start_project(project_name):
    """
    创建项目
    :param project_name: 项目名称
    """
    ProjectGenerator().project_generate(project_name)


@click.command('swagger2api')
@click.option('-s', '--swagger-doc',
              required=True, help='Swagger doc file.')
@click.option('-d', '--api-dir',
              required=False, help='api save dir.')
def swagger2api(swagger_doc, api_dir):
    """
    swagger文档转换成api-object
    :param swagger_doc: swagger.json 文件
    :param api_dir:  api存放路径 非必填
    """
    SwaggerGenerator().generate(swagger_doc, api_dir)


@click.command('run')
@click.option('-c', '--testcase',
              required=True, help='testcase')
@click.option('-t', '--tag',
              required=False, help='testcase')
@click.option('-r', '--reset', help='auto clear report', required=False, default='false',
              type=click.Choice(['true', 'false']))
def run(testcase, tag, reset):
    """
    运行用例并集成allure报告
    :param testcase: 用例路径
    :param tag: 用例标签
    :param reset: 是否重置allure报告
    :return: 
    """""
    command = f'pytest -v -s {testcase}'
    if reset == 'true':
        if 'win' in sys.platform:
            subprocess.call(f'rmdir /Q /S allure-results', shell=True)
        else:
            subprocess.call(f'rmdir -rf allure-results', shell=True)
    if tag:
        command += f' -m {tag}'
    subprocess.call(command + ' --alluredir=./allure-results', shell=True)


@click.command('har2case')
@click.option('-h', '--har',
              required=True, help='har file path')
@click.option('-a', '--api',
              required=False, help='api object dir', default=None)
@click.option('-t', '--testcase', help='testcase dir', required=False, default='testcase')
@click.option('-e', '--exclude', help='exclude url', required=False, default='')
def har2case(har, api, testcase, exclude):
    """
    将har转换成用例
    :param har: har文件路径
    :param api: har对应api-object文件
    :param testcase:  用例路径
    :param exclude:  过来的url
    :return:
    """
    hp = HarParser(har_file_path=har, api_object=api, exclude_url=exclude)
    hp.generate_testcase(testcase_path=testcase)


group.add_command(start_project)
group.add_command(swagger2api)
group.add_command(run)
group.add_command(har2case)


def cmd():
    group.main()


if __name__ == '__main__':
    cmd()



# -*- coding:utf-8 -*-
# @Time     :2023/2/3 4:05 下午
# @Author   :CHNJX
# @File     :testcase_mixin.py
# @Desc     :testcase 通用方法

from jsonschema import validate

import json
import logging
import os

from api_driver.utils.custom_str_utils import CustomStrUtils
from api_driver.utils.fake import Fake
from api_driver.utils.placeholder import Placeholder


class TestcaseMixin:
    logger: logging.Logger

    # 将字典中的形参转换成实参
    def replace_formal_dict_2_actual(self, formal_data: dict):
        for key, value in formal_data.items():
            if type(value) is str:
                formal_data[key] = self.replace_formal_str_2_actual(value)
            elif type(value) is dict:
                self.replace_formal_dict_2_actual(value)
            elif type(value) is list:
                self.replace_formal_list_2_actual(value)

    # 将列表中的形参转成实参
    def replace_formal_list_2_actual(self, formal_list: list):
        for i in range(len(formal_list)):
            i_value = formal_list[i]
            if type(i_value) is str:
                formal_list.insert(i, self.replace_formal_str_2_actual(i_value))
            elif type(i_value) is list:
                self.replace_formal_list_2_actual(i_value)
            elif type(i_value) is dict:
                self.replace_formal_dict_2_actual(i_value)

    # 字符串形参替换
    def replace_formal_str_2_actual(self, formal_str):
        if "${random" in formal_str:
            self.logger.info(f'对 {formal_str} 进行占位符替换')
            formal_map = {}
            # 对随机数形参进行替换
            random_range = CustomStrUtils.get_random_num(formal_str)
            if type(random_range) is int:
                # 获取到了random范围 生成随机数
                random_num = Fake.get_random_string(random_range)
                # 将形参和实参存到map中
                formal_map['random(' + str(random_range) + ')'] = random_num
            elif type(random_range) is list:
                random_num = Fake.get_range_random(int(random_range[0]), int(random_range[1]))
                # 将形参和实参存到map中
                formal_map['random(' + str(random_range[0]) + ',' + str(random_range[1]) + ')'] = random_num

            else:
                self.logger.error('随机数占位符替换失败：' + formal_str)
                return formal_str
            return Placeholder.resolve_str(formal_str, formal_map)
        elif "${timeStamp}" in formal_str:
            # 获取时间戳
            ts = Fake.get_time_stamp()
            return Placeholder.resolve_str(formal_str, {'timeStamp': ts})
        else:
            return formal_str

    def assert_schema(self, res, schema_file_dir, schema_file):
        schema = json.loads(open(os.path.join(schema_file_dir, schema_file), 'r', encoding='utf-8').read())
        validate(res, schema)



# -*- coding:utf-8 -*-
# @Time     :2023/2/2 10:41 下午
# @Author   :CHNJX
# @File     :project_generator.py
# @Desc     :项目创造器

import sys
from os import listdir
from os.path import dirname, exists, join

sys.path.append(dirname(sys.path[0]))

from api_driver.tenplate import Template
from api_driver import ad_utils


class ProjectGenerator:
    def project_generate(self, project_name):
        """
        创建项目
        :param project_name: 项目名称
        :return: None
        """
        if exists(project_name):
            print(f'project {project_name} is already existed')
            return 1
        ad_utils.create_folder(project_name)
        ad_utils.create_folder(join(project_name, 'testcase'))
        ad_utils.create_folder(join(project_name, 'swagger'))
        ad_utils.create_folder(join(project_name, 'har'))
        ad_utils.create_folder(join(project_name, 'api_object'))
        for dir_name in listdir(project_name):
            if dir_name == 'swagger' or dir_name == 'har':
                continue
            cur_dir = join(project_name + '/' + dir_name, '__init__.py')
            ad_utils.write("", cur_dir)
        self.__generate_base_need(project_name)

    def __generate_base_need(self, project_name):
        template = Template()
        api_object_dir = join(project_name, 'api_object')
        testcase_dir = join(project_name, 'testcase')
        ad_utils.write(template.get_content('http.tpl'), join(api_object_dir, 'base_api.py'))
        ad_utils.write(template.get_content('api_demo.tpl'), join(api_object_dir, 'api_demo.py'))
        ad_utils.write(template.get_content('base_testcase.tpl'), join(testcase_dir, 'test_base.py'))
        ad_utils.write(template.get_content('testcase_demo.tpl'), join(testcase_dir, 'testcase_demo.py'))
        ad_utils.write(swagger_json, join(f'{project_name}/swagger', 'swagger_demo.json'))


swagger_json = r"""
{
  "swagger": "2.0",
  "info": {
    "version": "1.0.0",
    "title": "Sample API",
    "description": "A sample API for demonstration purposes",
    "contact": {
      "name": "Your Name",
      "email": "youremail@example.com"
    }
  },
  "host": "api.example.com",
  "basePath": "/v1",
  "schemes": [
    "http"
  ],
  "tags": [
    {
      "name": "users",
      "description": "Operations related to users"
    }
  ],
  "paths": {
    "/users": {
      "get": {
        "tags": [
          "users"
        ],
        "summary": "Get a list of users",
        "description": "Returns a list of all users in the system",
        "produces": [
          "application/json"
        ],
        "responses": {
          "200": {
            "description": "Successful operation",
            "schema": {
              "type": "array",
              "items": {
                "$ref": "#/definitions/User"
              }
            }
          },
          "default": {
            "description": "Error response",
            "schema": {
              "$ref": "#/definitions/Error"
            }
          }
        }
      }
    },
    "/users/{userId}": {
      "get": {
        "tags": [
          "users"
        ],
        "summary": "Get a user by ID",
        "description": "Returns the user with the specified ID",
        "produces": [
          "application/json"
        ],
        "parameters": [
          {
            "name": "userId",
            "in": "path",
            "description": "The ID of the user to retrieve",
            "required": true,
            "type": "integer"
          }
        ],
        "responses": {
          "200": {
            "description": "Successful operation",
            "schema": {
              "$ref": "#/definitions/User"
            }
          },
          "default": {
            "description": "Error response",
            "schema": {
              "$ref": "#/definitions/Error"
            }
          }
        }
      }
    }
  },
  "definitions": {
    "User": {
      "type": "object",
      "properties": {
        "id": {
          "type": "integer"
        },
        "name": {
          "type": "string"
        },
        "email": {
          "type": "string"
        }
      }
    },
    "Error": {
      "type": "object",
      "properties": {
        "message": {
          "type": "string"
        },
        "code": {
          "type": "integer"
        }
      }
    }
  }
}
"""

if __name__ == '__main__':
    pass



# -*- coding:utf-8 -*-
# @Time     :2022/6/29 10:55
# @Author   :CHNJX
# @File     :placeholder.py
# @Desc     :形参占位符转成实参

PLACEHOLDER_PREFIX = '${'
PLACEHOLDER_SUFFIX = '}'


class Placeholder:
    # 声明占位符的前后缀
    @classmethod
    def resolve_str(cls, text: str, parameter: dict):
        """
        转换字符串中携带的占位符
        """
        if parameter is None or len(parameter) == 0 or text is None or text == '':
            return text
        # 计算占位符的起始位置
        start_index = text.find(PLACEHOLDER_PREFIX)
        while start_index != -1:
            # 获取占位符最后的索引
            end_index = text.find(PLACEHOLDER_SUFFIX, start_index + len(PLACEHOLDER_PREFIX))
            if end_index != -1:
                # 取出要替换的变量名
                formal = text[start_index + len(PLACEHOLDER_PREFIX):end_index]
                # 获取占位符后的索引  一遍往后遍历 替换所有占位符
                next_index = end_index + len(PLACEHOLDER_SUFFIX)
                try:
                    actual = str(parameter[formal])
                    if actual:
                        # 替换占位符
                        text = text.replace('${' + formal + '}', actual)
                        next_index = start_index + len(actual)
                except Exception as e:
                    raise ValueError(
                        "Could not resolve placeholder '" + formal + "' in [" + text + "]: " + e.__str__())
                # 替换一次后 查看后续位置是否存在占位符  继续替换
                start_index = text.find(PLACEHOLDER_PREFIX, next_index)
            else:
                start_index = -1
        return text

    @classmethod
    def resolve_dict(cls, dic: dict, parameter: dict):
        """替换字典中值的占位符"""
        if parameter is None or len(parameter) == 0 or dic is None or len(dic) == 0:
            return dic
        res = {}
        for key, value in dic.items():
            if PLACEHOLDER_PREFIX in value:
                res[key] = cls.resolve_str(value, parameter)
        return res



import os
from os.path import join, dirname

from api_driver.har_parser import HarParser
from api_driver.project_generator import ProjectGenerator
from api_driver.swagger_generate import SwaggerGenerator


class TestCommand:
    def test_start_project(self):
        ProjectGenerator().project_generate('new_project')
        assert os.path.isdir(os.path.join('new_project', "api_object"))
        assert os.path.isdir(os.path.join('new_project', "testcase"))

    def test_swagger_generate(self):
        SwaggerGenerator().generate(join(dirname(__file__), 'swagger/swagger.yaml'),
                                    join(dirname(__file__), 'api_object'))
        assert os.path.exists(os.path.join(dirname(__file__) + "/api_object", "users.py"))

    def test_har2case_generate(self):
        har_file = join(dirname(__file__), 'data/demo.har')
        testcase_dir = join(dirname(__file__), 'testcase')
        har = HarParser(har_file_path=har_file)
        har.generate_testcase(testcase_path=testcase_dir)
        assert os.path.exists(os.path.join(dirname(__file__) + "/testcase", "test_demo.py"))


# -*- coding:utf-8 -*-
# @Time     :2023/2/7 12:45
# @Author   :CHNJX
# @File     :tenplate.py
# @Desc     :模板工具类
import os

from jinja2 import FileSystemLoader, Environment


class Template:
    def __init__(self):
        loader = FileSystemLoader(os.path.join(
            os.path.dirname(__file__), 'templates'))
        self.env = Environment(loader=loader)

    def get_content(self, tpl_name, **kwargs):
        return self.env.get_template(tpl_name).render(**kwargs)



# -*- coding:utf-8 -*-
# @Time     :2023/2/1 3:22 下午
# @Author   :CHNJX
# @File     :swagger_generate.py
# @Desc     :将swagger转换成api-object
import os
import re
import sys
from os.path import dirname, exists, join

sys.path.append(dirname(sys.path[0]))

from api_driver.loader_swagger import load_swagger
from api_driver.tenplate import Template
from api_driver import ad_utils


class SwaggerGenerator:

    def generate(self, swagger_doc, api_dir=None):
        """
        将swagger 装换成api-object
        :param swagger_doc: swagger文档
        :param api_dir:     api-object存放路径
        :return:
        """
        if not api_dir:
            api_dir = 'api_object'
        if '/' not in swagger_doc and '\\' not in swagger_doc:
            swagger_doc = 'swagger/' + swagger_doc
        swagger_data = load_swagger(swagger_doc)
        self._generate_template_path(swagger_data['paths'])
        tag_path_dict = self._generate_template_data(swagger_data)
        template = Template()
        for tag, paths in tag_path_dict.items():
            content = template.get_content('api.tpl', tag=tag, paths=paths)
            file_path = os.path.join(api_dir, tag.lower() + '.py')
            ad_utils.write(content, file_path)

    def _get_http_method(self, value: dict) -> str:
        """
        提取请求方式
        :param value: get:{} or post:{} or put:{} ...
        :return: request method: get/post/put/delete...
        """
        attribute = ''
        if value.get('get'):
            attribute = 'get'
        elif value.get('post'):
            attribute = 'post'
        elif value.get('put'):
            attribute = 'put'
        elif value.get('delete'):
            attribute = 'delete'
        return attribute

    def _transformation_parameters(self, parameters) -> list:
        """
        parameters dict => parameters list
        """
        return [param for param in parameters if
                param['name'] != 'raw' and param['name'] != 'root' and param['in'] != 'header']

    # 转换json参数
    def _transformation_json(self, parameters) -> list:
        """
        将json参数装换成json参数名称列表
        :return: [param1，param2...]
        """
        json_list = []
        for param in parameters:
            if param.get('schema') and param['schema'].get('properties'):
                for key_name in param['schema']['properties'].keys():
                    json_list.append(key_name)
        return json_list

    # 拼接参数列表
    def _transformation_params_list(self, params, json_params: list) -> list:
        params_name_list: list = [param['name'] for param in params]
        params_name_list.extend(json_params)
        return params_name_list

    def _transform_url(self, path, method):
        """对url进行转换 适应restful风格"""
        path_name_list: list[str] = path.split('/')
        pat = re.compile(r'[@_!#$%^&*()<>?/\|}{~:]')
        res = {}
        if 'id' in path_name_list[-1].lower() or pat.search(path_name_list[-1]):
            res['url_param'] = re.sub('[\W_]+', '', path_name_list[-1])
            if method == 'get':
                res['name'] = 'detail'
            elif method == 'post':
                res['name'] = 'add'
            elif method == 'put':
                res['name'] = 'update'
            elif method == 'delete':
                res['name'] = 'delete'
            path_name_list.pop(-1)
            res['url'] = '/'.join(path_name_list)
        else:
            res['url_param'] = ''
            res['url'] = path
            res['name'] = path_name_list[-1].split('?')[0]
        return res

    def _transformation_file(self, json_params) -> list:
        """抽取文件类型参数"""
        return [param for param in json_params if 'file' in param]

    def _generate_template_path(self, swagger_paths):
        """
        对path是进行解析
        :param swagger_paths: swagger_docs 解析的path字典
        :return:
        """
        for path, value in swagger_paths.items():
            method_attribute = self._get_http_method(value)
            value['method'] = method_attribute
            value['tag'] = value[method_attribute]['tags'][0]
            value['desc'] = value[method_attribute]['summary']
            parameters = value[method_attribute]['parameters'] if value[method_attribute].get('parameters') else ''
            value['parameters'] = self._transformation_parameters(parameters)
            value['json'] = self._transformation_json(parameters)
            value['files'] = self._transformation_file(value['json'])
            value['params_list'] = self._transformation_params_list(value['parameters'], value['json'])
            value.update(self._transform_url(path, method_attribute))

    def _generate_template_data(self, swagger_data) -> dict:
        """将数据改造后存入字典"""
        tag_path_dict = {tag['name'].replace('/', '-', -1).capitalize():
                             {name: path for name, path in swagger_data['paths'].items()
                              if path['tag'].replace('/', '-', -1) == tag['name'].replace('/', '-', -1)}
                         for tag in swagger_data['tags']}

        return tag_path_dict



# -*- coding:utf-8 -*-
# @Time     :2023/2/28 18:30
# @Author   :CHNJX
# @File     :har_parser.py
# @Desc     :har parse code
import base64
import json
import logging
import os
import re
import sys
from json import JSONDecodeError
from urllib.parse import urlparse

from api_driver.tenplate import Template
from api_driver import ad_utils

# 忽略的请求头
IGNORE_REQUEST_HEADERS = [
    "host",
    "accept",
    "content-length",
    "connection",
    "accept-encoding",
    "accept-language",
    "origin",
    "referer",
    "cache-control",
    "pragma",
    "cookie",
    "upgrade-insecure-requests",
    ":authority",
    ":method",
    ":scheme",
    ":path",
    "sec-ch-ua",
    'sec-ch-ua-platform',
    'sec-fetch-dest',
    'sec-fetch-mode',
    'sec-fetch-site',
    'x-requested-with',
    'sec-ch-ua-mobile'
]


class HarParser:

    def __init__(self, har_file_path: str, exclude_url: str = None, api_object: str = None):
        if '/' not in har_file_path and '\\' not in har_file_path:
            har_file_path = 'swagger/' + har_file_path
        self.har_file = har_file_path
        self.exclude_url = exclude_url or ""
        self.api_class = ''
        self.func_name = ''
        self.module_dir = []
        self.api_object = api_object.replace('\\', '/', -1) if api_object else None

    def load_har_2_entry_json(self) -> list[dict]:
        """
        load har data
        :return:
            list: entries
            [
                {
                    "request": {},
                    "response": {}
                },
                {
                    "request": {},
                    "response": {}
                }
            ]
        """
        with open(self.har_file, 'r+', encoding='utf-8') as f:
            try:
                entry_json = json.loads(f.read())
                return entry_json['log']['entries']
            except (KeyError, TypeError):
                sys.exit(1)

    def __make_step_validate(self, step_dict, entry_json):
        """
        解析har数据 组装断言需要的数据
        :return:
            {
                "validate":[
                    {"equals": ['key1','expected1']}
                    {"in": ['key2','expected2']}
                ]
            }
        """
        # 对响应状态码进行断言
        step_dict["validate"].append(
            {"equals": ["status_code", entry_json["response"].get("status")]}
        )

        resp_content_dict = entry_json['response'].get('content')

        # 对请求头进行断言
        headers = ad_utils.covert_list_to_dict(entry_json['response'].get('headers', []))
        if 'Content-Type' in headers:
            step_dict["header_validate"].append(
                {"equals": ["Content-Type", headers["Content-Type"]]}
            )

        text = resp_content_dict.get("text")
        if not text:
            return
        mime_type: str = resp_content_dict.get("mimeType")
        # 根据编码格式对text内容进行解码
        if mime_type and mime_type.startswith('application/json'):
            encoding = resp_content_dict.get('encoding')
            if encoding and encoding == 'base64':
                content = base64.b64decode(text).decode('utf-8')
            else:
                content = text
            try:
                resp_content_json = json.loads(content)
            except JSONDecodeError:
                logging.warning(
                    "响应无法转换成json格式: {}".format(content.encode("utf-8"))
                )
                return

            if not isinstance(resp_content_json, dict):
                return
            for key, value in resp_content_json.items():
                if isinstance(value, (dict, list)):
                    continue
                if key == 'timestamp':
                    continue
                step_dict["json_validate"].append(
                    {"equals": [key, value]}
                )

    def __make_request_data(self, step_dict: dict, entry_json: dict):
        """
        将har中请求数据进行封装
        :param entry_json:
            entry_json (dict):
                {
                    "request": {
                        "method": "POST",
                        "postData": {
                            "mimeType": "application/x-www-form-urlencoded; charset=utf-8",
                            "params": [
                                {"name": "a", "value": 1},
                                {"name": "b", "value": "2"}
                            }
                        },
                    },
                    "response": {...}
                }

        :returns:
            {
                "request": {
                    "method": "POST",
                    "data": {"v": "1", "w": "2"}
                }
            }
        """
        method = entry_json['request'].get('method')
        # 判断请求是否通过body进行传输
        if method in ['POST', 'PUT', 'PATCH']:
            post_data = entry_json['request'].get('postData', {})
            mimetype: str = post_data.get('mimeType')

            if 'text' in post_data:
                data = post_data.get('text')
            else:
                params = post_data.get('params', [])
                data = ad_utils.covert_list_to_dict(params)
            request_data_key = 'data'
            # 判断是表单还是json
            if not mimetype:
                pass
            elif mimetype.startswith("application/json"):
                try:
                    data = json.loads(data)
                    request_data_key = 'json'
                except JSONDecodeError:
                    pass
            elif mimetype.startswith("application/x-www-form-urlencoded"):
                data = ad_utils.convert_x_www_form_to_dict(data)

            step_dict['request'][request_data_key] = data

    def __make_request_method(self, step_dict, entry_json):
        """
        获取har中的请求方式
        """
        method = entry_json["request"].get("method")
        if not method:
            logging.exception("method missed in request.")
            sys.exit(1)
        step_dict['request']['method'] = method

    def __make_request_headers(self, step_dict: dict, entry_json: dict):
        """
        获取har中对应header数据
        :param entry_json:
           entry_json (dict):
               {
                   "request": {
                       "headers": [
                           {"name": "Host", "value": "httprunner.top"},
                           {"name": "Content-Type", "value": "application/json"},
                           {"name": "User-Agent", "value": "iOS/10.3"}
                       ],
                   },
                   "response": {}
               }

           :return:
               {
                   "request": {
                       headers: {"Content-Type": "application/json"}
               }

        """
        step_headers = {}
        for header in entry_json['request'].get('headers', []):
            if header['name'].lower() in IGNORE_REQUEST_HEADERS:
                # 直接忽略不进行处理
                continue
            step_headers[header['name']] = header['value']
        if step_headers:
            step_dict['request']['headers'] = step_headers

    def __make_request_url(self, step_dict: dict, entry_json: dict):
        """
        将json数据中url的信息进行提取
        :param entry_json:
            entry_json (dict):
                    {
                        "request": {
                            "url": "https://httprunner.top/home?v=1&w=2",
                            "queryString": [
                                {"name": "v", "value": "1"},
                                {"name": "w", "value": "2"}
                            ],
                        },
                        "response": {}
                    }

        :return:
            {
                "name: "/home",
                "request": {
                    url: "https://httprunner.top/home",
                    params: {"v": "1", "w": "2"}
                }
            }
        """
        # 将url携带的参数进行分离
        url_params = ad_utils.covert_list_to_dict(entry_json["request"].get("queryString", []))
        url = entry_json['request'].get('url')
        if not url:
            logging.exception('请求缺少url信息')
            sys.exit(1)
        # 将url进行拆解
        parse_url = urlparse(url)
        if url_params:
            parse_url = parse_url._replace(query='')  # 清除url上的参数
            step_dict["request"]["url"] = parse_url.geturl()
            step_dict["request"]["params"] = url_params
        else:
            step_dict["request"]["url"] = parse_url.geturl()
        step_dict['name'] = parse_url.path

    def _gen_step(self, entry_json) -> dict:
        """
        将entry_json 转换成测试步骤需要用的的数据
        :param entry_json:
            entry_json (dict):
                    {
                        "request": {
                            "method": "POST",
                            "url": "https://httprunner.top/api/v1/Account/Login",
                            "headers": [],
                            "queryString": [],
                            "postData": {},
                        },
                        "response": {
                            "status": 200,
                            "headers": [],
                            "content": {}
                        }
                    }
        :return:
        """
        step_dict = {
            "name": "",
            "request": {},
            "validate": [],
            "json_validate": [],
            "header_validate": []
        }
        self.__make_request_url(step_dict, entry_json)
        self.__make_request_headers(step_dict, entry_json)
        self.__make_request_method(step_dict, entry_json)
        self.__make_request_data(step_dict, entry_json)
        self.__make_step_validate(step_dict, entry_json)

        return step_dict

    def generate_testcase_steps(self, fmt_version) -> list:
        """
        封装测试用例步骤
        :param fmt_version:
        :return:
        """

        def is_exclude_url(url, exclude_url: str) -> bool:
            """查看是否为剔除的url"""
            exclude_url_list = exclude_url.split('|')
            for exclude in exclude_url_list:
                if exclude and exclude in url:
                    return True
            return False

        entry_json = self.load_har_2_entry_json()
        test_steps = []
        # 去除不需要的url
        for entry in entry_json:
            url = entry["request"].get("url")
            if is_exclude_url(url, self.exclude_url):
                continue
            test_steps.append(self._gen_step(entry))
        return test_steps

    def _make_testcase(self, case_name: str, fmt_version: str) -> str:
        """
        :param fmt_version: 用例版本
        :return: 通过模板对用例转换后的内容
        """
        # 先把需要的用例步骤进行解析
        testcase_steps = self.generate_testcase_steps(fmt_version)
        testcase = {
            'model_name': case_name.capitalize(),
            'case_name': case_name,
            'testcase_steps': testcase_steps
        }
        temp = Template()
        if self.api_object:
            self.generate_api_case(testcase_steps)
            testcase['module_dir_list'] = set(self.module_dir)
            return temp.get_content('har2api_case.tpl', **testcase)
        return temp.get_content('har2case.tpl', **testcase)

    def find_object_func(self, url: str, method: str) -> dict:
        """
        根据url和请求方法获取到对应的接口
        :param url: 接口路径
        :param method: 请求方法
        :return: {'api_class': self.api_class, 'func_name': self.func_name, 'module_dir': self.module_dir}
        """
        url_list = url.split('/')
        url = '/'.join([url_list[-2], url_list[-1]])
        if not os.path.exists(self.api_object):
            logging.exception('api-object 路径有误')
            sys.exit(1)
        self.travel_api_object(self.api_object, url, method)
        api_func = {'api_class': self.api_class, 'func_name': self.func_name}
        self.api_class = ''
        self.func_name = ''
        return api_func

    def travel_api_object(self, api, url, method):
        """遍历目录获取对应的api"""
        if self.api_class or self.func_name:
            return
        if os.path.isfile(api):
            api = api.replace('\\', '/', -1)
            self.api_class, self.func_name = ad_utils.get_class_and_func(api, url, method)
            if self.api_class and self.func_name:
                file_path = re.search('api_object/(.*?).py', api).group(1)
                # 获取包路径
                file_path_list = file_path.split('/') if '/' in file_path else [file_path]
                self.module_dir.append('api_object.' + '.'.join(file_path_list))

        elif os.path.isdir(api):
            for api_file in os.listdir(api):
                os.path.join(api, api_file)
                self.travel_api_object(os.path.join(api, api_file), url, method)

    def generate_api_case(self, testcase_steps):
        """
        转换成api_object 模式的用例步骤
        :param testcase_steps:
        :return:
        """
        for step in testcase_steps:
            url = step['request']['url']
            method = step['request']['method']
            step.update(self.find_object_func(url, method))

    def generate_testcase(self, fmt_version: str = 'v1', testcase_path: str = 'testcase'):
        """
        将har文件转换成测试用例
        :param testcase_path: 用例存放地址 默认放在testcase目录 需要存在与TestBase统一目录
        :param fmt_version: 用例版本
        """
        # 1 先获取文输出文件名
        # 切割文件和扩展名
        har_file_dir = os.path.splitext(self.har_file)[0]

        logging.info('开始转换测试用例')
        case_name = ''
        if '/' in har_file_dir:
            case_name = har_file_dir.split('/')[-1]
        elif '\\' in har_file_dir:
            case_name = har_file_dir.split('\\')[-1]
        testcase_path = os.path.join(testcase_path, 'test_'+case_name + '.py')
        testcase_content = self._make_testcase(case_name, fmt_version)
        ad_utils.write(testcase_content, testcase_path)
        logging.info(f'完成{har_file_dir}的用例转换')


if __name__ == '__main__':
    pass



# -*- coding:utf-8 -*-
# @Time     :2022/6/28 20:05
# @Author   :CHNJX
# @File     :fake.py
# @Desc     :造假数据
import random
import string
import time
from datetime import datetime

from faker import Faker


class Fake:
    fake = Faker(locale="zh-CN")

    @classmethod
    def get_range_random(cls, num1, num2):
        return random.Random().randint(num1, num2)

    @classmethod
    def get_random_int(cls, num):
        num1 = 1 * (num - 1)
        num2 = int('9' * num)
        return random.Random().randint(num1, num2)

    @classmethod
    def get_time_stamp(cls):
        return int(time.time())

    @classmethod
    def get_random_string(cls, str_len: int) -> str:
        return "".join(
            random.choice(string.ascii_letters + string.digits) for _ in range(str_len)
        )

    @classmethod
    def get_current_date(cls) -> str:
        return str(datetime.today().date())

    @classmethod
    def get_current_datetime(cls) -> str:
        return datetime.now().strftime('%Y-%m-%d %H:%M:%S')
    
    @classmethod
    def get_random_name(cls):
        """
        获取随机中文名
        :return:
        """
        return cls.fake.name()

    @classmethod
    def get_random_phone(cls):
        """获取随机手机号"""
        return cls.fake.phone_number()

    @classmethod
    def get_random_ssn(cls):
        """获取随机身份证号"""
        return cls.fake.ssn()

    @classmethod
    def get_random_credit_card_number(cls):
        """获取随机银行卡号"""
        return cls.fake.credit_card_number()

    @classmethod
    def get_random_address(cls):
        """获取随机地址"""
        return cls.fake.address().split(' ')[0]

    @classmethod
    def get_random_email(cls):
        """获取随机邮箱"""
        return cls.fake.email()

    @classmethod
    def get_random_date(cls):
        """获取随机日期"""
        return cls.fake.date()

    @classmethod
    def get_random_this_year_date(cls):
        """获取今年随机日期"""
        return cls.fake.date_this_year()

    @classmethod
    def get_random_date_time(cls):
        """获取随机时间"""
        return cls.fake.date_time()

    @classmethod
    def get_random_this_year_date_time(cls):
        """获取随机时间"""
        return cls.fake.date_time_this_year()

    @classmethod
    def get_random_future_date(cls):
        """获取未来的日期"""
        return cls.fake.future_date()

    @classmethod
    def get_random_future_date_time(cls):
        """获取未来的日期"""
        return cls.fake.future_datetime()

    @classmethod
    def get_customer_date(cls, start_date: str, end_date: str):
        """
        获取自定义范围的日期
        :param start_date:  开始时间 1y为往后一年  -1y为往前一年
        :param end_date:  结束时间
        :return:
        """
        return cls.fake.date_between(start_date=start_date, end_date=end_date)

    @classmethod
    def get_customer_date_time(cls, start_date: str, end_date: str):
        """
        获取自定义范围的时间
        :param start_date:  开始时间 +1y为往后一年  -1y为往前一年
        :param end_date:  结束时间
        :return:
        """
        return cls.fake.date_time_between(start_date=start_date, end_date=end_date)



# -*- coding:utf-8 -*-
# @Time     :2022/6/29 17:12
# @Author   :CHNJX
# @File     :custom_str_utils.py
# @Desc     :字符串的自定义方法
import random
import re


class CustomStrUtils:

    @classmethod
    def get_random_num(cls, formal_str):
        """
        获取随机数占位符的范围
        """
        if "random" not in formal_str:
            raise ValueError(
                f'The random placeholder was not found in the incoming parameter: {formal_str}, expect:random(num) or random(num1,num2)')

        # 匹配上 random中的范围
        try:
            random_range = re.match(r'.*{random\((.*?)\)}', formal_str, flags=0).group(1)
            if ',' in random_range:
                return random_range.split(',')
            return int(random_range)
        except Exception as e:
            raise ValueError(
                f'Wrong random number placeholder format：{formal_str}, expected:random(num) or random(num1,num2)')


if __name__ == '__main__':
    CustomStrUtils.get_random_num('{random(1}')



# -*- coding:utf-8 -*-
# @Time     :2023/3/1 11:08
# @Author   :CHNJX
# @File     :ad_utils.py
# @Desc     :public utils
import ast
from os import makedirs
import os
from urllib.parse import unquote
import re


def covert_list_to_dict(list_data):
    """
    将har list data 转换成dict
    :param list_data
        list_data (list)
            [
                {"name": "v", "value": "1"},
                {"name": "w", "value": "2"}
            ]

    :return:
        dict:
            {"v": "1", "w": "2"}
    """
    return {item['name']: item.get('value') for item in list_data}


def convert_x_www_form_to_dict(form_data: str):
    """
    将form表单数据装换成dict字典数据
    :param form_data:  (str): a=1&b=2
    :return: (dict) {"a":1, "b":2}
    """
    if isinstance(form_data, str):
        res_dict = {}
        for kev_value in form_data.split('&'):
            try:
                key, value = kev_value.split('=')
            except ValueError:
                raise Exception(f"错误的 x_www_form_urlencoded 数据: {form_data}")
            # unquote('abc%20def') -> 'abc def'.
            res_dict[key] = unquote(value)
        return res_dict
    else:
        return form_data


def get_class_and_func(api_file, url, method) -> tuple:
    """
    获取到类名和方法名
    :param api_file: 对应api文件
    :param url:     对应请求url
    :param method:  请求方式
    :return:
    """
    api_class = ''
    func_name = ''
    with open(api_file, 'r', encoding='utf-8') as f:
        cd = ast.parse(f.read())
        for item in cd.body:
            if isinstance(item, ast.ClassDef) and item.body:
                for func in item.body:
                    if isinstance(func, ast.FunctionDef):
                        func_str = ast.dump(func)
                        if url in func_str and method.lower() in func_str:
                            pattern = "name='(.*?)'"
                            api_class = re.search(pattern, ast.dump(item)).group(1)
                            func_name = re.search(pattern, ast.dump(func)).group(1)
                            break
    return api_class, func_name


def write(content, file_path):
    """
    将内容写入文档中
    :param content:  要写入的内容
    :param file_path: 文件路径（不存在则会进行创建）
    :return: None
    """
    dir_ = os.path.dirname(file_path)
    if not os.path.exists(dir_):
        os.makedirs(dir_)
    with open(file_path, 'w', encoding='utf-8') as f:
        f.write(content)


def create_folder(path):
    """
    创建目录
    :param path: 目录路径
    :return: None
    """
    makedirs(path)
    print(f'create folder {path}')







from __future__ import absolute_import
import re
import ast
from setuptools import setup

_version_re = re.compile(r'__version__\s+=\s+(.*)')

with open('api_driver/_version.py', 'rb') as f:
    version = str(ast.literal_eval(_version_re.search(
        f.read().decode('utf-8')).group(1)))

with open('requirements.txt') as f:
    requirements = [line for line in f.read().splitlines() if line]

setup(
    name='apidriver',
    description='api test framework cli',
    version=version,
    author='CHNJX',
    author_email='360088940@qq.com',
    url='https://github.com/JxcChen/service-driver',
    packages=['api_driver','api_driver.utils'],
    package_data={'templates': ['api_driver/templates/*']},
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'adf=api_driver:cmd'
        ]
    },
    install_requires=requirements,
    tests_require=['pytest'],
)



# -*- coding:utf-8 -*-
# @Time     :2023/1/30 6:32 下午
# @Author   :CHNJX
# @File     :loader_swagger.py
# @Desc     :
import json
from os import path, listdir
from os.path import isdir

import six
import yaml


def get_loader(filename):
    if filename.endswith('.json'):
        loader = json.load
    elif filename.endswith('.yml') or filename.endswith('.yaml'):
        loader = yaml.load
    else:
        with open(filename, 'r', 'utf-8') as f:
            contents = f.read()
            contents = contents.strip()
            if contents[0] in ['{', '[']:
                loader = json.load
            else:
                loader = yaml.load
    return loader


def modify_spec_data(field, spec_data, data):
    if not isinstance(spec_data, dict) or not isinstance(data, dict):
        return None
    for k, v in data.items():
        if k in spec_data[field]:
            spec_data[field][k].update(v)
        else:
            spec_data[field][k] = v


def get_ref_filepath(filename, ref_file):
    ref_file = path.normpath(path.join(path.dirname(filename), ref_file))
    return ref_file


def load_file(filename, spec_data):
    loader = get_loader(filename)
    with open(filename, 'r', encoding='utf-8') as f:
        if filename.endswith('.json'):
            data = loader(f)
        else:
            data = loader(f, yaml.Loader)
        # modify_spec_data(spec_data, data)
        spec_data.update(data)
        for field, values in six.iteritems(data):
            if field not in ['definitions', 'parameters', 'paths']:
                continue
            if not isinstance(values, dict):
                continue
            for _field, value in six.iteritems(values):
                # _field is the endpoint for paths when the api of paths contains $ref
                if _field == '$ref' and value.endswith('.yml'):
                    _filepath = get_ref_filepath(filename, value)
                    field_data = load_swagger(_filepath)
                    # modify_spec_data(field, spec_data, field_data)
                    spec_data[field] = field_data
                elif '$ref' in value:
                    v = value.pop('$ref', '')
                    if not v:
                        continue
                    _filepath = get_ref_filepath(filename, v)
                    field_data = load_swagger(_filepath)
                    modify_spec_data(field, spec_data, field_data)


def load_swagger(filename):
    spec_data = {}
    files = listdir(filename) if isdir(filename) else [filename]
    for f in files:
        if f != filename:
            f = filename + '/' + f
        load_file(f, spec_data)
    return spec_data



# -*- coding:utf-8 -*-
# @Time     :2023/2/3 7:48 上午
# @Author   :CHNJX
# @File     :service_logger.py
# @Desc     :日志构造器

import logging
import os
import sys
import time
from datetime import datetime


class Logger:

    @classmethod
    def getLogger(cls, name, root_path, package_name="log") -> logging:
        logger = logging.getLogger(name)
        if logger.handlers:
            return logger
        logger.setLevel(logging.DEBUG)
        now_string = datetime.now().strftime('%Y%m%d')
        file_name = f'{now_string}_log.log'
        logger_dir = os.path.join(root_path, package_name)
        if not os.path.exists(logger_dir):
            os.mkdir(logger_dir)
        file_path = os.path.join(logger_dir, file_name)
        t = int(time.time())

        # FileHandler
        fh = logging.FileHandler(file_path, mode='a', encoding='utf-8')
        fh.setLevel(logging.DEBUG)
        formatter = logging.Formatter(
            '******%(asctime)s - %(name)s - %(filename)s,line %(lineno)s - %(levelname)s: %(message)s')
        fh.setFormatter(formatter)

        sh = logging.StreamHandler(sys.stdout)
        sh.setFormatter(formatter)
        sh.setLevel(logging.DEBUG)
        logger.addHandler(fh)
        logger.addHandler(sh)

        return logger
